# Poisson regression using D. virilis , D. melanogaster, and D. willistoni  data   v. Nov 7 2021
# part of the ms "New genes in the Drosophila Y chromosome: lessons from D. willistoni "  J. Ricchio, F. Uno, and AB Carvalho 

#implements the Poisson regression test for the equality of the gain and loss rates
#data file structure: 
#branchlength, events, gainloss, branch_name


# dataset <- "AF_vir_mel_wil"	#Assumption-free (D. virilis + D. melanogaster + D. willistoni)
dataset <- "AF_vir_mel_wil_conservative"	# Assumption-free (D. virilis + D. melanogaster + D. willistoni), removing 3 recent gains in the willistoni Y

if (dataset == "AF_vir_mel_wil" )   { bias <- 1.000 ;d<-read.table("AF_vir_mel_wil_data.txt", header=T, sep=",")} 
if (dataset == "AF_vir_mel_wil_conservative" )   { bias <- 1.000 ;d<-read.table("AF_vir_mel_wil_conservative_data.txt", header=T, sep=",")}

names(d)
attach(d)

factor(gainloss)

gainloss_test2<-glm(events ~ offset(log(branchlength)) + gainloss, family=poisson)
summary(gainloss_test2)

#the following line test for the fit of the data to a Poisson distribuition
1 - pchisq(deviance(gainloss_test2), df.residual(gainloss_test2))


coef_regr <- coefficients(summary(gainloss_test2))
beta1 <-  coef_regr[2,1]           #this is the Poisson regression coefficient for "gainloss"
Std_Err_beta1 <- coef_regr[2,2]    #this is Std. Error of the Poisson regression coefficient for "gainloss"
z <- beta1 / Std_Err_beta1         #this is the standard test for Ho = 0
adjusted_z <- (abs(beta1) - log(bias)) / Std_Err_beta1   #this is the test for Ho = log(bias)

noquote (c("P value for gain/loss ratio <> ", bias , 2*(1-pnorm(adjusted_z))))

gain_loss_ratio <- exp(abs(beta1))
noquote (c("gain/loss ratio = ", gain_loss_ratio ))
adjusted_beta1 <- (abs(beta1) - log(bias))
adjusted_gain_loss_ratio <- exp(adjusted_beta1)
noquote (c("unbiased gain/loss ratio = ", adjusted_gain_loss_ratio ))
noquote (c("95% confidence interval for the unbiased gain/loss ratio:"))
exp( adjusted_beta1 - 1.96 * Std_Err_beta1 )
exp( adjusted_beta1 + 1.96 * Std_Err_beta1 )


